.allianceauth-buyback-program-plugin .buy-value-True{
	color: green !important;

}

.allianceauth-buyback-program-plugin .tooltip {
  position: relative;
  display: inline-block;
  border-bottom: 1px dotted black;
}

.allianceauth-buyback-program-plugin .tooltip .tooltiptext {
  visibility: hidden;
  width: 120px;
  background-color: black;
  color: #fff;
  text-align: center;
  border-radius: 6px;
  padding: 5px 0;

  /* Position the tooltip */
  position: absolute;
  z-index: 1;
  bottom: 100%;
  left: 50%;
  margin-left: -60px;
}

.allianceauth-buyback-program-plugin .tooltip:hover .tooltiptext {
  visibility: visible;
}

.allianceauth-buyback-program-plugin .item-icon{
  height: 25px !important;
  padding-right: 5px;
}
